
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 neuron <neuron@hollowtube.mine.nu>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA.
 */

#ifndef INITNG_NGCS_H
#define INITNG_NGCS_H

#include "../../src/initng_active_db.h"
#include "../../src/initng_system_states.h"
#include "../../src/initng_list.h"
#include "ngcs_common.h"

#include <stddef.h>

typedef struct ngcs_cmd_s ngcs_cmd;
typedef struct ngcs_channel_s ngcs_channel;
typedef struct ngcs_conn_s ngcs_conn;
typedef struct ngcs_request_s ngcs_request;

/* int module_init(const char *version);
   void module_unload(void); */

int service_status(active_db_h * service);
void is_system_halt(h_sys_state state);

/*! \brief Close the specified channel
 *  Called to close a channel. Sends a message with length -1 on the channel to 
 *  notify the client (if the connection is still open) and frees the ngcs_channel
 *  structure, calling the ngcs_channel.free callback first if there is one
 *
 *  \param chan the channel to be closed
 */
void ngcs_close_channel(ngcs_channel * chan);

/* \brief Register an ngcs command
 * Registers an ngcs command. ngcs clients can then call it by sending a 
 * suitably-structured message on channel 0, specifying the command name. If 
 * more than one registered command has the same name, the results are undefined.
 *
 * \param cmd structure describing the command - must not be freed, modified 
 *        or passed to ngcs_reg_cmd again until ngcs_unreg_cmd(cmd) has been called
 * \sa ngcs_unreg_cmd() ngcs_cmd
 */
void ngcs_reg_cmd(ngcs_cmd * cmd);

/* \brief Unregister an ngcs command
 * Unregisters a command registered with ngcs_reg_cmd.
 *
 * \param cmd the ngcs_cmd structure passed to ngcs_reg_cmd
 * \sa ngcs_reg_cmd()
 */
void ngcs_unreg_cmd(ngcs_cmd* cmd);
int ngcs_channel_send(ngcs_channel * chan, int type, int len, char *data);
ngcs_channel *ngcs_open_channel(ngcs_conn * conn,
                                void (*gotdata) (ngcs_channel *, int, int,
                                                 char *),
                                void (*chanfree) (ngcs_channel *));
int ngcs_send_response(ngcs_request * req, int type, int len, char *data);

/*! \brief Ngcs command handler
 *
 *  Represents an ngcs command handler. Ngcs commands are sent on channel 0 of
 *  a connection and are identified by a (case-sensitive) string, their name 
 *
 *  \sa ngcs_reg_cmd() ngcs_unreg_cmd()
 */
struct ngcs_cmd_s
{
    /*! \brief Name of the command */
    const char *name;
    
    /*! \brief Callback function to handle the request
     *
     *  The callback function to handle the request. It should call  ngcs_send_response
     *  exactly once to send a response to the request.
     *  \sa ngcs_send_response()
     */
    void (*func) (ngcs_request * req);

    /* \brief List head - internal use */
    struct list_head list;
};

struct ngcs_channel_s
{
    int id;
    ngcs_conn *conn;
    void *user_data;
    void (*gotdata) (ngcs_channel *, int /* type */ , int /* len */ ,
                     char * /* data */ );
    void (*free) (ngcs_channel *);
    struct list_head list;
};

struct ngcs_conn_s
{
    f_module_h fdw;
    ngcs_channel chans;
    int nextid;
    struct list_head list;
};

struct ngcs_request_s
{
    int type;
    int len;
    char *data;
    ngcs_conn *conn;
    int sent_resp_flag;
};

extern ngcs_conn ngcs_conns;
extern ngcs_conn ngcs_dead_conns;
extern ngcs_cmd ngcs_cmds;

#define while_ngcs_conns(current) list_for_each_entry_prev(current, &ngcs_conns.list, list)
#define while_ngcs_conns_safe(current, safe) list_for_each_entry_prev_safe(current, safe, &ngcs_conns.list, list)

#define while_ngcs_dead_conns(current) list_for_each_entry_prev(current, &ngcs_dead_conns.list, list)
#define while_ngcs_dead_conns_safe(current, safe) list_for_each_entry_prev_safe(current, safe, &ngcs_dead_conns.list, list)

#define while_ngcs_chans(current, conn) list_for_each_entry_prev(current, &(conn)->chans.list, list)
#define while_ngcs_chans_safe(current, safe, conn) list_for_each_entry_prev_safe(current, safe, &(conn)->chans.list, list)

#define while_ngcs_cmds(current) list_for_each_entry_prev(current, &ngcs_cmds.list, list)
#define while_ngcs_cmds_safe(current, safe) list_for_each_entry_prev_safe(current, safe, &ngcs_cmds.list, list)

#endif /* !INITNG_NGCS_H */
